/*
 * VarStar - Search & report variable star records in AAVSO data file
 * Copyright (C) 2005-2007  Frederick (Rick) Burton
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

// VarStarGUIDlg.cpp : implementation file
//

#include "stdafx.h"
#include "VarStarGUI.h"
#include "VarStarGUIDlg.h"
#include ".\varstarguidlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	enum { IDD = IDD_ABOUTBOX };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

// Implementation
protected:
	DECLARE_MESSAGE_MAP()
public:
	//afx_msg void OnFileLoadfile();
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//ON_COMMAND(ID_FILE_LOADFILE, OnFileLoadfile)
END_MESSAGE_MAP()


// CVarStarGUIDlg dialog


// Constructor setting default values used in edits
// ----------------------------------------------------------------------------
CVarStarGUIDlg::CVarStarGUIDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CVarStarGUIDlg::IDD, pParent)
	, m_MinRaHr(0)
	, m_MaxRaHr(23)
	, m_SouthDeg(-90)
	, m_NorthDeg(90)
	, m_GreatestMinMag(0)
	, m_LeastMaxMag(6)
	, m_MinDays(0)
	, m_MaxDays(1000)
	, m_bool_specific(FALSE)
	, m_SearchResults(_T(""))
	, m_fileName(_T(""))
	, m_records_loaded(0)
	, m_records_searched(0)
	, m_bool_magnitude(TRUE)
	, m_bool_period(TRUE)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CVarStarGUIDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);

	DDX_Text(pDX, IDC_EDIT1, m_MinRaHr);
	DDV_MinMaxInt(pDX, m_MinRaHr, 0, 23);
	DDX_Text(pDX, IDC_EDIT2, m_MaxRaHr);
	DDV_MinMaxInt(pDX, m_MaxRaHr, 0, 23);
	DDX_Text(pDX, IDC_EDIT3, m_SouthDeg);
	DDV_MinMaxInt(pDX, m_SouthDeg, -90, 90);
	DDX_Text(pDX, IDC_EDIT4, m_NorthDeg);
	DDV_MinMaxInt(pDX, m_NorthDeg, -90, 90);
	DDX_Text(pDX, IDC_EDIT5, m_LeastMaxMag);
	DDV_MinMaxFloat(pDX, m_LeastMaxMag, 0.0, 99.0);
	DDX_Text(pDX, IDC_EDIT6, m_GreatestMinMag);
	DDV_MinMaxFloat(pDX, m_GreatestMinMag, 0.0, 99.0);
	DDX_Text(pDX, IDC_EDIT7, m_MinDays);
	DDV_MinMaxDouble(pDX, m_MinDays, 0.0, 100000.0);
	DDX_Text(pDX, IDC_EDIT8, m_MaxDays);
	DDV_MinMaxDouble(pDX, m_MaxDays, 0.0, 100000.0);

	DDX_Control(pDX, IDC_EDIT1, m_CEdit_MinRA);
	DDX_Control(pDX, IDC_EDIT2, m_CEdit_MaxRA);
	DDX_Control(pDX, IDC_EDIT3, m_CEdit_MinDeg);
	DDX_Control(pDX, IDC_EDIT4, m_CEdit_MaxDeg);
	DDX_Control(pDX, IDC_EDIT5, m_CEdit_MaxMag);
	DDX_Control(pDX, IDC_EDIT6, m_CEdit_MinMag);
	DDX_Control(pDX, IDC_EDIT7, m_CEdit_MinDays);
	DDX_Control(pDX, IDC_EDIT8, m_CEdit_MaxDays);

	DDX_Control(pDX, IDC_EDIT9, m_CEdit_FileName);
	DDX_Control(pDX, IDC_LIST2, m_StarList);

	DDX_Control(pDX, IDC_CHK_MAGNITUDE, m_chk_magnitude);
	DDX_Control(pDX, IDC_CHK_PERIOD,    m_chk_period);
	DDX_Control(pDX, IDC_SPECIFIC,      m_chk_specific);
	DDX_Check(pDX, IDC_CHK_MAGNITUDE, m_bool_magnitude);
	DDX_Check(pDX, IDC_CHK_PERIOD,    m_bool_period);
	DDX_Check(pDX, IDC_SPECIFIC,      m_bool_specific); // DO WE NEED THIS LINE?

	DDX_Control(pDX, IDC_BUTTON1, m_Btn_Search);
	DDX_Control(pDX, IDC_BUTTON2, m_Btn_SearchDesc);

	DDX_Text(pDX, IDC_EDIT9,  m_fileName);
	DDX_Text(pDX, IDC_EDIT10, m_records_loaded);
	DDX_Text(pDX, IDC_EDIT11, m_records_searched);
}

BEGIN_MESSAGE_MAP(CVarStarGUIDlg, CDialog)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
ON_BN_CLICKED(IDC_BUTTON1, OnBnClickedSearch)
ON_BN_CLICKED(IDC_BUTTON2, OnBnClickedSearchDesc)
ON_BN_CLICKED(IDC_SPECIFIC, OnBnClickedSpecific)
ON_COMMAND(ID_FILE_SAVECSV, OnFileSavecsv)
ON_COMMAND(ID_FILE_SAVETXT, OnFileSavetxt)
ON_COMMAND(ID_FILE_LOADFILE, OnFileLoadfile)
ON_COMMAND(ID_HELP_ABOUTVARSTARGUI, OnHelpAboutVarStar)
ON_BN_CLICKED(IDC_CHK_MAGNITUDE, OnBnClickedChkMagnitude)
ON_BN_CLICKED(IDC_CHK_PERIOD, OnBnClickedChkPeriod)
END_MESSAGE_MAP()


// CVarStarGUIDlg message handlers

BOOL CVarStarGUIDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// Setting up list column headings and widths
	// ------------------------------------------------------------------------
	m_StarList.InsertColumn(0, "R.A.",        LVCFMT_RIGHT,  40);
	m_StarList.InsertColumn(1, "Dec.",        LVCFMT_RIGHT,  35);
	m_StarList.InsertColumn(2, "Name",        LVCFMT_RIGHT,  75);
	m_StarList.InsertColumn(3, "Type",        LVCFMT_LEFT,   70);
	m_StarList.InsertColumn(4, "Magnitude",   LVCFMT_LEFT,   95);
	m_StarList.InsertColumn(5, "Period",      LVCFMT_RIGHT,  75);
	m_StarList.InsertColumn(6, "Description", LVCFMT_LEFT,  120);

	m_init = true;
	//m_chk_specific.SetCheck(0);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CVarStarGUIDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CVarStarGUIDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CVarStarGUIDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

// Load variable star records validating them on-the-fly
// --------------------------------------------------------------------------------------

// File Menu "Load Data..."
void CVarStarGUIDlg::OnFileLoadfile()
{
	// Clear the grid
	m_StarList.DeleteAllItems();

	int r = DoFileDlg(TRUE, "txt", "*.txt",
		"Text Files (*.txt)|*.txt|All Files (*.*)|*.*||");

	if(r > 0) // Records validated and loaded, enable controls
	{
		this->EnableLocCtrls();
		this->EnablePropCtrls();
		this->EnableSrchCtrls();

		//this->m_chk_specific.SetCheck(-1);
		this->m_CEdit_MinRA.SetFocus();
		this->m_CEdit_MinRA.SetSel(0, 2);
		this->m_records_loaded = r;
		this->m_records_searched = 0;
	}
	else // Error loading, disable controls
	{
		this->EnableLocCtrls(FALSE);
		this->EnablePropCtrls(FALSE);
		this->EnableSrchCtrls(FALSE);
		this->m_records_loaded = 0;
		this->m_records_searched = 0;
		this->m_fileName = "";

		r == 0 ? // Return code 0?, bad load, else bad file
			AfxMessageBox("No records loaded! Verify data file contains valid variable star records.") :
			AfxMessageBox("Data File Error!");
	}

	UpdateData(FALSE);
}


// Event handler functions
// --------------------------------------------------------------------------------------

// "Magnitude" search check box
void CVarStarGUIDlg::OnBnClickedChkMagnitude()
{
	if(this->m_chk_magnitude.GetCheck() == 0)
		this->EnableMagCtrls(FALSE);
	else
		this->EnableMagCtrls(TRUE);
}

// "Period" search check box
void CVarStarGUIDlg::OnBnClickedChkPeriod()
{
	if(this->m_chk_period.GetCheck() == 0)
		this->EnablePerCtrls(FALSE);
	else
		this->EnablePerCtrls(TRUE);
}

// "No Light Curve" search check box
void CVarStarGUIDlg::OnBnClickedSpecific()
{
	if(this->m_chk_specific.GetCheck() == 0)
		this->EnablePropCtrls(TRUE);
	else
		this->EnablePropCtrls(FALSE);
}

// Ascending search button
void CVarStarGUIDlg::OnBnClickedSearch()
{
	this->DisplaySearch();
}

// Descending search button
void CVarStarGUIDlg::OnBnClickedSearchDesc()
{
	this->DisplaySearch(false);
}

// File Menu "Save CSV..."
void CVarStarGUIDlg::OnFileSavecsv()
{
	if(DoFileDlg(FALSE, "csv", "*.csv",
		"Comma-Separated Files (*.csv)|*.csv|All Files (*.*)|*.*||") <= 0)
		AfxMessageBox("No records saved! Verify data loaded and searched by criteria.");
}

// File Menu "Save TXT..."
void CVarStarGUIDlg::OnFileSavetxt()
{
	if(DoFileDlg(FALSE, "txt", "*.txt",
		"Text Files (*.txt)|*.txt|All Files (*.*)|*.*||") <= 0)
		AfxMessageBox("No records saved! Verify data loaded and searched by criteria.");
}

// Help Menu "About vStar"
void CVarStarGUIDlg::OnHelpAboutVarStar()
{
	CAboutDlg abtDlg;
	abtDlg.DoModal();
}


// Special functions called by event handlers
// --------------------------------------------------------------------------------------

// Universal File Dialog wrapper
int CVarStarGUIDlg::DoFileDlg(BOOL opensave, const char* ext1, const char* ext2,
							   const char* filters)
{
	int r = -99;
	// Create an Open/Save dialog
	CFileDialog fileDlg(opensave, ext1, ext2, OFN_HIDEREADONLY, filters, this);

	// Display the file dialog. User clicks OK?
	if( fileDlg.DoModal() == IDOK )
	{
		CString fileName = fileDlg.GetPathName();

		opensave == TRUE ? // Open?, load data
			r = x.loadRecords(fileName.GetString()) :
			fileDlg.GetFileExt() == "csv" ? // Save?, which format?
				r = x.saveReport(fileName.GetString()) : // *.csv
				r = x.saveFile(fileName.GetString());    // *.txt

		opensave == TRUE ? // Open?, set the file name for display
			this->m_fileName = fileDlg.GetFileName() :
			this->m_fileName = "";
	}

	return r;
}

// Enable / disable location controls
void CVarStarGUIDlg::EnableLocCtrls(BOOL enable)
{
	this->m_CEdit_MinRA.EnableWindow(enable);
	this->m_CEdit_MaxRA.EnableWindow(enable);
	this->m_CEdit_MinDeg.EnableWindow(enable);
	this->m_CEdit_MaxDeg.EnableWindow(enable);
}

// Enable / disable property controls
void CVarStarGUIDlg::EnablePropCtrls(BOOL enable)
{
	if(!enable)
		this->EnableMagCtrls(FALSE);
	else if(this->m_chk_magnitude.GetCheck() == 0)
		this->EnableMagCtrls(FALSE);
	else
		this->EnableMagCtrls(TRUE);
	this->m_chk_magnitude.EnableWindow(enable);

	if(!enable)
		this->EnablePerCtrls(FALSE);
	else if(this->m_chk_period.GetCheck() == 0)
		this->EnablePerCtrls(FALSE);
	else
		this->EnablePerCtrls(TRUE);
	this->m_chk_period.EnableWindow(enable);
}

// Enable / disable magnitude controls
void CVarStarGUIDlg::EnableMagCtrls(BOOL enable)
{
	this->m_CEdit_MaxMag.EnableWindow(enable);
	this->m_CEdit_MinMag.EnableWindow(enable);
}

// Enable / disable period controls
void CVarStarGUIDlg::EnablePerCtrls(BOOL enable)
{
	this->m_CEdit_MaxDays.EnableWindow(enable);
	this->m_CEdit_MinDays.EnableWindow(enable);
}

// Enable / disable search controls
void CVarStarGUIDlg::EnableSrchCtrls(BOOL enable)
{
	this->m_Btn_Search.EnableWindow(enable);
	this->m_Btn_SearchDesc.EnableWindow(enable);
	this->m_chk_specific.EnableWindow(enable);
}

// Places field values in the grid in ascending or descending order
void CVarStarGUIDlg::DisplaySearch(bool ascending) // default ascending = true
{
	UpdateData(TRUE);

	int i, j, deg;
	double days;
	int nIndex;
	char strDataBuf[25];

	// Set search criteria values
	// ------------------------------------------------------------------------
	x.setSchMinHr(m_MinRaHr);
	x.setSchMaxHr(m_MaxRaHr);
	x.setSchMinDeg(m_SouthDeg);
	x.setSchMaxDeg(m_NorthDeg);
	x.setSchMinMag(m_GreatestMinMag);
	x.setSchMaxMag(m_LeastMaxMag);
	x.setSchMinDays(m_MinDays);
	x.setSchMaxDays(m_MaxDays);

	m_records_searched = x.searchRecords(this->m_bool_specific  ? false : true,
										 this->m_bool_magnitude ? true : false,
										 this->m_bool_period    ? true : false);

	m_StarList.DeleteAllItems();

	// Format and display field values ascending or descending
	i = 0;
	ascending ? x.moveFirst() : x.moveLast();
	while(ascending ? !x.eof() : !x.bof())
	{
		j = 1;

		sprintf(strDataBuf, "%02d:%02d", x.getHour(), x.getMin());
		nIndex = m_StarList.InsertItem(i++, strDataBuf);

		deg = x.getDeg();
		if(deg < 0)
			sprintf(strDataBuf, "%03d", deg);
		else
			sprintf(strDataBuf, "%02d", deg);
		m_StarList.SetItemText(nIndex, j++, strDataBuf);

		m_StarList.SetItemText(nIndex, j++, x.getName());
		m_StarList.SetItemText(nIndex, j++, x.getType());
		m_StarList.SetItemText(nIndex, j++, x.getRange());

		days = x.getDays();
		if(days != 0.0)
			sprintf(strDataBuf, "%10.4f", days);
		else
			sprintf(strDataBuf, "");
		m_StarList.SetItemText(nIndex, j++, strDataBuf);

		m_StarList.SetItemText(nIndex, j++, x.getDesc());

		ascending ? x.moveNext() : x.movePrev();
	}

	UpdateData(FALSE);
}
