/*
 * VarStar - Search & report variable star records in AAVSO data file
 * Copyright (C) 2005-2007  Frederick (Rick) Burton
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

// VarStarGUIDlg.h : header file
//

#pragma once
#include "afxwin.h"
#include "CStarRecords.h"
#include "afxcmn.h"


// CVarStarGUIDlg dialog
class CVarStarGUIDlg : public CDialog
{
// Construction
public:
	CVarStarGUIDlg(CWnd* pParent = NULL);	// standard constructor

// Dialog Data
	enum { IDD = IDD_VARSTARGUI_DIALOG };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support


// Implementation
protected:
	HICON m_hIcon;

	// Generated message map functions
	virtual BOOL OnInitDialog();
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	DECLARE_MESSAGE_MAP()

private:
	bool m_init;
	StarRecords x;

	// These vars contain search criteria
	// ------------------------------------------------------------------------
	short int m_MinRaHr;
	short int m_MaxRaHr;
	short int m_SouthDeg;
	short int m_NorthDeg;

	float  m_LeastMaxMag;
	float  m_GreatestMinMag;
	double m_MinDays;
	double m_MaxDays;

	// Check box states
	// ------------------------------------------------------------------------
	BOOL m_bool_magnitude;
	BOOL m_bool_period;
	BOOL m_bool_specific;

	// Record counts
	// ------------------------------------------------------------------------
	int m_records_loaded;
	int m_records_searched;

	CString m_fileName;
	CString m_SearchResults;

	// Control variables
	// ------------------------------------------------------------------------
	CEdit m_CEdit_FileName;
	CListCtrl m_StarList;

	CEdit m_CEdit_MinRA;
	CEdit m_CEdit_MaxRA;
	CEdit m_CEdit_MinDeg;
	CEdit m_CEdit_MaxDeg;

	CEdit m_CEdit_MaxMag;
	CEdit m_CEdit_MinMag;
	CEdit m_CEdit_MaxDays;
	CEdit m_CEdit_MinDays;

	CButton m_Btn_Search;
	CButton m_Btn_SearchDesc;
	CButton m_chk_specific;
	CButton m_chk_magnitude;
	CButton m_chk_period;

	// Event handlers
	// ------------------------------------------------------------------------
	afx_msg void OnBnClickedChkMagnitude();
	afx_msg void OnBnClickedChkPeriod();
	afx_msg void OnBnClickedSpecific();

	afx_msg void OnBnClickedSearch();
	afx_msg void OnBnClickedSearchDesc();

	afx_msg void OnFileSavecsv();
	afx_msg void OnFileSavetxt();
	afx_msg void OnFileLoadfile();
	afx_msg void OnHelpAboutVarStar();

	// Helper functions called by event handlers
	// ------------------------------------------------------------------------
	void DisplaySearch(bool ascending = true);
	void EnableLocCtrls(BOOL enable = TRUE);
	void EnablePropCtrls(BOOL enable = TRUE);
	void EnableSrchCtrls(BOOL enable = TRUE);
	void EnableMagCtrls(BOOL enable = TRUE);
	void EnablePerCtrls(BOOL enable = TRUE);
	int DoFileDlg(BOOL opensave, const char* ext1, const char* ext2, 
		const char* filter);

};
